package server;
import java.io.*;
import java.net.*;
import java.util.Vector;

public class ChatServer extends Thread {
	public static final int	 DEFAULT_PORT = 1223;
	int	port = 0;
	ServerSocket	server;
	Socket socket;
	Vector clients = new Vector();	
	public ChatServer(int port) {
		if (port == 0) port = DEFAULT_PORT;
		this.port = port;
		try {
			server = new ServerSocket(port);
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		System.out.println("ChatServer: listening on port " + port);
		this.start();
	}

	public void run() {
		
		while(true) {
			try {
				socket = server.accept();
				System.out.println("Client Request for connection accepted!");
			} catch(IOException e) {
				System.out.println("Exception while listening client connections");
				break;
			}
			clients.addElement(socket);
			try {
				new MsgBroadCaster(this).start();
			} catch(IOException e) {
				System.out.println("Couldn't open stream to Client");
			}
			Thread.yield();
		}
	}
	
	public static void main(String[] args) {
		int	port = 0;
		if (args.length == 1) {
			try {
				port = Integer.parseInt(args[0]);
			}catch (NumberFormatException e) {
				port = 0;
			}
		}
		new ChatServer(port);
	}
	public void closeServer() throws IOException {
		server.close();
		Socket sock;
		for(int i = 0; i < clients.size(); i++) {
			sock = (Socket) clients.elementAt(i);
			sock.close();
		}
	}
}


class MsgBroadCaster extends Thread {
	private ChatServer server;
	private Socket	socket_to_myClient;
    	private boolean stop;
    	private DataInputStream	stream_from_client;
	private Vector allClients;
	public MsgBroadCaster(ChatServer server) throws IOException {
		this.server = server;
		this.socket_to_myClient = server.socket;
		this.allClients = server.clients;
		this.stream_from_client = new DataInputStream(socket_to_myClient.getInputStream());
	}
    
    	public void run() {
    		String	line = null;
    		while(true) {
			try {
				line = stream_from_client.readUTF();
                	} catch(IOException e) {
                		try {
                			if(socket_to_myClient != null) { 
                				stream_from_client.close();
                				socket_to_myClient.close();
                				allClients.removeElement(socket_to_myClient);
                				System.out.println(allClients.size());//for testing use!
                				break;
                			}
                		} catch(Exception ex) {
                			e.printStackTrace();
                		}
                	}
                	System.out.println(line);//for testing use!!
                	
                	for (int i = 0; i < allClients.size(); i++){
                		try {
                			
                			Socket client = (Socket)allClients.elementAt(i);
                			DataOutputStream toClient = new DataOutputStream(client.getOutputStream());
                			toClient.writeUTF(line);
                		} catch(IOException e) {
                			System.out.println("One of the clients has been out");
                			allClients.removeElementAt(i);
                			
                		}
                	}//end of for loop 
		} // end of while loop  
	}
}
